/*
 * stepper.c
 *
 * Created: 2/17/2018 5:39:34 PM
 *  Author: Jake
 */ 

#include "stepper.h"
#include "hardware.h"

stepper_t stepper_new(pin_t *step_pin, pin_t *dir_pin){
	stepper_t stepper;
	
	stepper.step_pin = step_pin;
	stepper.dir_pin = dir_pin;
	
	stepper_reset(&stepper);
	
	return stepper;
}

void stepper_reset(stepper_t *stepper){
	
	stepper->one_minute = 160980000; // calib'd # for timer ticks per minute (speeds, accels all in steps/min or steps/min/s)
	stepper->speed_period = 0;
	stepper->accel_period = 0;
	
	stepper->speed = 0;
	
	stepper->last_step = 0;
	stepper->last_accel = 0;
	
	stepper->position_ticks = 0;
	stepper->position_accel_to = 0;
	stepper->position_deccel_from = 0;
	stepper->position_ticks_end = 0;
}

void stepper_steps(stepper_t *stepper, int32_t steps, uint32_t speed){
	// set speed period
	(speed < 1) ? speed = 1 : (0); // avoid 0 division, 1 step / s seems like reasonable lower bound step rate
	stepper->speed = speed;
	stepper->speed_period = stepper->one_minute / stepper->speed;
	
	// set direction
	if(steps < 0){
		pin_clear(stepper->dir_pin);
	} else {
		pin_set(stepper->dir_pin);
	}
		
	// set new position information
	stepper->position_ticks_end = abs(steps);
	stepper->position_accel_to = 0;
	stepper->position_deccel_from = stepper->position_ticks_end; // defines flat line block
	stepper->position_ticks = 0;
	
	// reset times
	TICKER_SYNC;
	unsigned long now = TICKER;
	stepper->last_accel = now;
	stepper->last_step = now;
}

void stepper_block(stepper_t *stepper, int32_t steps, uint32_t entryspeed, uint32_t accel, uint32_t accellength, uint32_t deccellength){
	// or, newblock: does maths and adds to queue
	
	// enforce no div/0
	(entryspeed < 1) ? entryspeed = 1 : (0);
	// going to have to catch blocks which cause deceleration to 0 during deceleration phases !
	stepper->speed = entryspeed;

	// do starting speed period
	stepper->speed_period = stepper->one_minute / stepper->speed;
	stepper->accel_period = stepper->one_minute / accel;
	
	// set dir
	if(steps < 0){
		pin_clear(stepper->dir_pin);
	} else {
		pin_set(stepper->dir_pin);
	}
	
	// do lengths
	stepper->position_ticks_end = abs(steps);
	stepper->position_accel_to = accellength;
	stepper->position_deccel_from = deccellength;
	stepper->position_ticks = 0;
	
	// reset times
	TICKER_SYNC;
	unsigned long now = TICKER;
	stepper->last_accel = now;
	stepper->last_step = now;
}

void stepper_update(stepper_t *stepper){
	if(stepper->position_ticks_end > stepper->position_ticks){ // still have somewhere to go
		TICKER_SYNC;
		unsigned long now = TICKER;
		
		if(stepper->position_ticks < stepper->position_accel_to){
			// we're accelerating!
			if(now - stepper->last_accel > stepper->accel_period){
				stepper->speed += 1;
				stepper->speed_period = stepper->one_minute / stepper->speed;
				stepper->last_accel = now;
			}
		} else if(stepper->position_ticks > stepper->position_deccel_from){
			if(now - stepper->last_accel > stepper->accel_period){
				stepper->speed -= 1;
				(stepper->speed < 1) ? stepper->speed = 1 : (0); // assert no 0's
				stepper->speed_period = stepper->one_minute / stepper->speed;
				stepper->last_accel = now;
			}
		} else {
			// no accel or deccel
		}
		
		// check if it's time to step
		if(now - stepper->last_step >= stepper->speed_period){
			pin_toggle(stepper->step_pin);
			stepper->position_ticks ++;
			stepper->last_step = now; //stepper->last_step + stepper->speed_period; // last speed_period if accelerating
			// not sure why that wasn't working: for now, take for granted that over the course of many steps,
			// we tend do equal amounts undershooting speed on all motors
		} // end step cycle
		
	}// end step if not there cycle
}